// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistable;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.taxeditor.editor.view.concept.ConceptViewPart;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveViewPart;
import eu.etaxonomy.taxeditor.editor.view.media.MediaViewPart;
import eu.etaxonomy.taxeditor.model.ContextListenerAdapter;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;

/**
 * <p>EditorStateManager class.</p>
 *
 * @author n.hoffmann
 * @created Sep 30, 2010
 * @version 1.0
 */
public class EditorStateManager extends ContextListenerAdapter {

	private static final String EDITORS = "editors";

	private static final String EDITOR = "editor";

	private TaxonEditorInputFactory inputFactory;
	
	/**
	 * <p>Constructor for EditorStateManager.</p>
	 */
	public EditorStateManager() {
		inputFactory = new TaxonEditorInputFactory();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextAboutToStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
	 */
	/** {@inheritDoc} */
	@Override
	public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
		saveEditorState(memento);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.ContextListenerAdapter#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
	 */
	/** {@inheritDoc} */
	@Override
	public void contextStop(IMemento memento, IProgressMonitor monitor) {
		EditorUtil.closeAll();
		resetConstantViews();
	}
	
	/**
	 * 
	 */
	private void resetConstantViews() {
		AbstractCdmEditorViewPart descriptiveView = (AbstractCdmEditorViewPart) EditorUtil.getView(DescriptiveViewPart.ID, false);
		AbstractCdmEditorViewPart mediaView = (AbstractCdmEditorViewPart) EditorUtil.getView(MediaViewPart.ID, false);
		AbstractCdmEditorViewPart conceptView = (AbstractCdmEditorViewPart) EditorUtil.getView(ConceptViewPart.ID, false);
		
		if(descriptiveView != null){
			descriptiveView.showEmptyPage();
		}
		if(mediaView != null){
			mediaView.showEmptyPage();
		}
		if(conceptView != null){
			conceptView.showEmptyPage();
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStart(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
	 */
	/** {@inheritDoc} */
	@Override
	public void contextStart(IMemento memento, IProgressMonitor monitor) {
		restoreEditors(memento, monitor);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.ContextListenerAdapter#contextRefresh(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void contextRefresh(IProgressMonitor monitor) {
		refreshEditors(monitor);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.ContextListenerAdapter#workbenchShutdown(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
	 */
	/** {@inheritDoc} */
	@Override
	public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
		saveEditorState(memento);
	}
	
	/**
	 * Reads taxon node UUIDs from the given memento and tries to open an editor 
	 * for each of the found UUIDs.
	 * 
	 * @param memento
	 */
	private void restoreEditors(final IMemento memento, IProgressMonitor monitor){
		if(memento == null){
			return;
		}
		
		IProgressMonitor subProgressMonitor = EditorUtil.getSubProgressMonitor(monitor, 1);
		
		IMemento editorMementos = memento.getChild(EDITORS);
		IMemento[] editorInputs = editorMementos.getChildren(EDITOR);
		
		subProgressMonitor.beginTask("Restoring Editors", 1 + editorInputs.length * 2);
		subProgressMonitor.subTask("Restoring Editors");
		subProgressMonitor.worked(1);
		
		for(IMemento editorInput : editorInputs){
			TaxonEditorInput input = (TaxonEditorInput) inputFactory.createElement(editorInput);
			subProgressMonitor.worked(1);
			if(!subProgressMonitor.isCanceled() && input != null){
				try {
					EditorUtil.open(input);
				} catch (PartInitException e) {
					MessagingUtils.error(getClass(), "Error opening an editor window", e);
				}
			}
			subProgressMonitor.worked(1);
		}
		subProgressMonitor.done();
	}
	
	/**
	 * @param monitor
	 */
	private void refreshEditors(IProgressMonitor monitor) {
		IProgressMonitor subProgressMonitor = EditorUtil.getSubProgressMonitor(monitor, 1);
		
		Set<IEditorPart> openEditors = EditorUtil.getOpenEditors();
		
		
		subProgressMonitor.beginTask("Refreshing Editors", openEditors.size() * 2);
		
		Set<UUID> uuids = new HashSet<UUID>();
		
		for(IEditorPart editor : openEditors){
			if(editor instanceof MultiPageTaxonEditor){
				try {
					uuids.add(((TaxonEditorInput) editor.getEditorInput()).getTaxonNode().getUuid());
				
//					((MultiPageTaxonEditor) editor).close(false);
					EditorUtil.close((MultiPageTaxonEditor)editor);
					subProgressMonitor.worked(1);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		for(UUID uuid : uuids){
			try {
				EditorUtil.openTaxonNode(uuid);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			subProgressMonitor.worked(1);
		}
		
		subProgressMonitor.done();
	}
	
	/**
	 * Saves the UUIDs 
	 * 
	 * @param memento
	 */
	private void saveEditorState(IMemento memento){
		if(memento == null){
			return;
		}
		
		Set<IEditorPart> editors = EditorUtil.getOpenEditors();
		
		IMemento editorMementos = memento.createChild(EDITORS);
		
		for(IEditorPart editor : editors){
			IEditorInput input = editor.getEditorInput();
			if(input instanceof IPersistable){
				IMemento editorMemento = editorMementos.createChild(EDITOR);
				((IPersistable) input).saveState(editorMemento);
			}
		}
	}
}
