// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Editor which shows a list of "data" that can be imported into the CDM
 * @author pplitzner
 * @date 20.09.2013
 *
 */
public abstract class DataImportEditor<T> extends EditorPart implements IPartContentHasFactualData, IDirtyMarkable,
IConversationEnabled, IPartContentHasDetails, IPartContentHasSupplementalData, IContextListener{

    protected Logger logger = Logger.getLogger(DataImportEditor.class);

    private ConversationHolder conversation;

    private TableViewer tableViewer;

    /**
     * Constructs a new DataImportEditor and registers it to listen to context changes
     */
    public DataImportEditor() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createPartControl(Composite parent) {

        Table table = new Table(parent, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
        table.setHeaderVisible(true);//TODO: set header title as query
        table.setLinesVisible(true);
        tableViewer = new TableViewer(table);

        // Propagate selection from viewer
        getSite().setSelectionProvider(tableViewer);

        //create context menu
        MenuManager menuManager = new MenuManager();
        getSite().registerContextMenu(menuManager, tableViewer);
        Control control = tableViewer.getControl();
        Menu menu = menuManager.createContextMenu(control);
        control.setMenu(menu);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
     */
    @Override
    public void setFocus() {
        tableViewer.getControl().setFocus();
        //make sure to bind again if maybe in another view the conversation was unbound
        if(!conversation.isBound()){
            conversation.bind();
        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.ui.texteditor.AbstractTextEditor#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
     */
    @Override
    public void init(IEditorSite site, IEditorInput input) throws PartInitException {
        CdmStore.getContextManager().addContextListener(this);
        setSite(site);
        setInput(input);
        setPartName(input.getName());
        setTitleToolTip(input.getName());
        conversation = ((DataImportEditorInput<T>)input).getConversationHolder();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor#dispose()
     */
    @Override
    public void dispose() {
        super.dispose();
        CdmStore.getContextManager().removeContextListener(this);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextAboutToStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
        //nothing
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
        //nothing
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStart(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        //nothing
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#contextRefresh(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void contextRefresh(IProgressMonitor monitor) {
        IEditorInput editorInput = getEditorInput();
        setInput(editorInput);
        if(editorInput instanceof DataImportEditorInput){
            tableViewer.getTable().removeAll();
            Collection<T> results = ((DataImportEditorInput<T>) editorInput).getResults();
            if(results!=null){
                for(T item:results){
                    TableItem tableItem = new TableItem(tableViewer.getTable(), SWT.NONE);
                    tableItem.setText(getTextForTableItem(item));
                    tableItem.setData(item);
                }
            }
        }
    }

    /**
     * Returns the text label of the given item.
     * @param item the item for which the label should be returnes
     * @return the label of the item
     */
    protected abstract String getTextForTableItem(T item);

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IContextListener#workbenchShutdown(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
        if(conversation!=null && conversation.isBound() && !conversation.isClosed()) {
            conversation.close();
        }
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
     */
    @Override
    public void update(CdmDataChangeMap changeEvents) {
        //nothing
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
     */
    @Override
    public ConversationHolder getConversationHolder() {
        return conversation;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#changed(java.lang.Object)
     */
    @Override
    public void changed(Object element) {
        // cannot be changed
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#forceDirty()
     */
    @Override
    public void forceDirty() {
        // cannot be dirty
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#doSaveAs()
     */
    @Override
    public void doSaveAs() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#isDirty()
     */
    @Override
    public boolean isDirty() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
     */
    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

}
